import io
from typing import (
    Dict,
    Union
)

class Emoji:
    def __init__(
        self, 
        state,
        data: Dict
        ) -> None:
        self._state = state
        
        self._raw: Dict = data
        self.id: int = data.pop('id')
        self.title: str = data.pop('title')
        self.slug: str = data.pop('slug')
        self.image: str = data.pop('image')
        self.url: str = self.image
        self.description: str = data.pop('description')
        self.category: int = data.pop('category')
        self.license: int = int(data.pop('license'))
        self.source: str = data.pop('source')
        self.faves: int = data.pop('faves')
        self.submitted_by: str = data.pop('submitted_by')
        self.width: int = data.pop('width')
        self.height: int = data.pop('height')
        self.filesize: int = data.pop('filesize')
        
    @property
    def formatted_description(self) -> str:
        """
        Returns
        -------
        str
            The emojis description formatted correctly with caps.
        """
        return self.description.capitalize()

    async def to_bytes(self) -> Union[io.BytesIO, None]:
        """
        |coro|
        
        Turn the image url to a bytes like object.
        
        Returns
        -------
            Union[io.BytesIO, None]
        """
        return await self._state.url_to_bytes(self.image)


import aiohttp

from typing import (
    Optional, 
    List, 
    Union,
    Dict,
    Any
)

from .http import HTTP
from .pack import Pack
from .emoji import Emoji


class Client:
    def __init__(
        self, 
        session: Optional[aiohttp.ClientSession] = None
    ) -> None:
        session = session or aiohttp.ClientSession()
        self._http = HTTP(session)
        
    async def fetch_emojis(self) -> List[Emoji]:
        """
        |coro|
        
        Retreives approx 5000 emojis from the website.
        
        Returns
        -------
        List[Emoji]
        """
        data = await self._http.fetch_emojis()
        return [Emoji(self._http, entry) for entry in data]
    
    async def fetch_packs(self) -> List[Pack]:
        """
        |coro|
        
        Retreives packs from the website.
        
        Returns
        -------
        List[Pack]
        """
        data = await self._http.fetch_packs()
        return [Pack(self._http, entry) for entry in data]
    
    async def fetch_statistics(self) -> Dict:
        """
        |coro|
        
        Retreives statistics about this website.
        
        Returns
        -------
        Dict
        """
        return await self._http.fetch_statistics()
    
    async def fetch_categories(self) -> Dict:
        """
        |coro|
        
        Retreives categories from the website. Fetches the current categories
        
        Returns
        -------
        Dict
        """
        return await self._http.fetch_categories()

    


from typing import List

class Pack:
    def __init__(
        self,
        state,
        data: dict
    ) -> None:
        self._state = state
        self._raw = data
        
        self.id: int = data.pop('id')
        self.name: str = data.pop('name')
        self.description: str = data.pop('description')
        self.slug: str = data.pop('slug')
        self.image: str = data.pop('image')
        self.url: str = self.image
        self.emojis: List[str] = data.pop('emojis')  # I want to change this.
        self.amount: int = data.pop('amount')
    
    @property
    def formatted_name(self) -> str:
        """
        Returns
        -------
        str
            The pack's name formatted correctly with caps.
        """
        return self.name.capitalize()

    @property
    def formatted_description(self) -> str:
        """
        Returns
        -------
        str
            The pack's description formatted correctly with caps.
        """
        return self.description.capitalize()
    
    


from typing import (
    Any,
    Union,
    Iterable,
    Callable
)

def get(
    iterable: Iterable,
    **kwargs
) -> Union[None, Any]:
    """
    Get something within a list according to your set attrs.

    Parameters
    ----------
    iterable: List[Any]
        The thing you want to iterate through
    *args: Any
        The args you want to check for.

    Examples
    --------
    Simple Example:
    ```python
    data = get(iterable, id=1234, title='This')
    ```
    """
    check = lambda entry: all((getattr(entry, key) == item for key, item in kwargs.items()))
    for iteration in iterable:
        if check(iteration):
            return iteration
    return None

def find(
    iterable: Iterable,
    check: Callable
) -> Union[None, Any]:
    """
    Find something within a list according to your check.
    
    Parameters
    ----------
    iterable: List[Any]
        The thing you want to iterate through
    check: Any
        The check func you want to use.
        
    Examples
    -------
    Using Lambda:
    ```python
    data = await client.fetch_emojis()
    specific_emoji = client.find(data, check=lambda emoji: emoji.name == 'this')
    ```
    
    Using a check func:
    ```python
    data = await client.fetch_emojis()
    
    def check(emoji):
        return emoji.name == 'this'
        
    specific_emoji = client.find(data, check=check)
    ```
    """
    for iter in iterable:
        if check(iter):
            return iter
    return None


import pathlib
from setuptools import setup

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

VERSION = "0.1.0"


setup(
    author="Iced Chai",
    name="emojigg",
    description="Async wrapper for the EmojiGG api.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/NextChai/Emojigg",
    project_urls = {
        'Documentation': 'https://github.com/NextChai/Emojigg/wiki',
        'Report a Bug': 'https://github.com/NextChai/Emojigg/issues',
        "Contribute": 'https://github.com/NextChai/Emojigg/pulls'
    },
    version=VERSION,
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    packages=["emojigg"],
    include_package_data=True,
    install_requires=[
        'aiofiles==0.7.0'
    ],
    entry_points={
        "console_scripts": [
            "square=square.__main__:main",
        ]
    },
)   